@@ -39,7 +39,7 @@ macro_rules! bind_params_sqlx_sqlite {
                     } else if value.is_date() {
                         query.bind(value.as_ref_date())
                     } else if value.is_time() {
-                        query.bind(value.as_ref_time().format("%T.f").to_string())
+                        query.bind(value.as_ref_time().map(|t| t.format("%T.f").to_string()))
                     } else if value.is_date_time() {
                         query.bind(value.as_ref_date_time())
                     } else if value.is_decimal() {
